# Copyright (c) 2016-2017  University of Houston

from __future__ import absolute_import

import os

from . import LocalService, InstallableApplication
from ..util import call


class ApacheHTTPD(LocalService):
  def __init__ (self, logh = None):
    super(ApacheHTTPD, self).__init__("apachehttpd", logh)

  def start_upstart (self):
    cmd = "sudo service apache2 start"
    self.call(cmd)

  def stop_upstart (self):
    cmd = "sudo service apache2 stop"
    self.call(cmd)


class Installer(InstallableApplication):
  HTTPDCONF = """Mutex file:/var/lock/apache2 default
PidFile /var/run/apache2/apache2.pid
User www-data
Group www-data
HostnameLookups Off
ErrorLog /var/log/apache2/error.log
LogFormat "%%h %%l %%u %%t \\"%%r\\" %%>s %%O \\"%%{Referer}i\\" \\"%%{User-Agent}i\\"" combined
CustomLog /var/log/apache2/access.log combined
LogLevel debug
LoadModule authz_core_module /usr/lib/apache2/modules/mod_authz_core.so
DocumentRoot /var/www/html
Listen %(serverip)s:80
LoadModule mpm_event_module /usr/lib/apache2/modules/mod_mpm_event.so
<IfModule mpm_event_module>
        StartServers             2
        MinSpareThreads          25
        MaxSpareThreads          75
        ThreadLimit              64
        ThreadsPerChild          25
        MaxRequestWorkers        150
        MaxConnectionsPerChild   0
</IfModule>
"""

  HTTPSSLCONF = """Mutex file:/var/lock/apache2 default
PidFile /var/run/apache2/apache2.pid
User www-data
Group www-data
HostnameLookups Off
ErrorLog /var/log/apache2/error.log
LogFormat "%%h %%l %%u %%t \\"%%r\\" %%>s %%O \\"%%{Referer}i\\" \\"%%{User-Agent}i\\"" combined
CustomLog /var/log/apache2/access.log combined
LogLevel debug
LoadModule authz_core_module /usr/lib/apache2/modules/mod_authz_core.so
DocumentRoot /var/www/html
Listen %(serverip)s:443
LoadModule mpm_event_module /usr/lib/apache2/modules/mod_mpm_event.so
<IfModule mpm_event_module>
        StartServers             2
        MinSpareThreads          25
        MaxSpareThreads          75
        ThreadLimit              64
        ThreadsPerChild          25
        MaxRequestWorkers        150
        MaxConnectionsPerChild   0
</IfModule>
LoadModule ssl_module /usr/lib/apache2/modules/mod_ssl.so
SSLEngine on
SSLCertificateFile "/etc/apache2/ssl/apache.crt"
SSLCertificateKeyFile "/etc/apache2/ssl/private.key"
"""
  CONFPATH = "/etc/apache2/apache2.conf"

  def __init__ (self, logh):
    super(Installer, self).__init__("apachehttpd", logh)

  def install_ubuntu (self):
    call("%s apache2" % (self.APT), self.logh)
    ApacheHTTPD().stop()

  def conf_ubuntu (self, serverip = None, ssl = False):
    if ssl:
      CURDIR = os.getcwd()
      os.chdir("/etc/apache2")
      try:
        os.mkdir("/etc/apache2/ssl")
      except OSError:
        self.logh.exception("/etc/apache2/ssl exists")
      os.chdir("/etc/apache2/ssl")
      call("sudo openssl req -nodes -newkey rsa:2048 -keyout private.key -out CSR.csr -subj '/C=US/ST=Texas/L=Houston/O=UH SDNLab/OU=KL/CN=%s'" % (serverip), self.logh)
      call("sudo openssl x509 -req -days 365 -in CSR.csr -signkey private.key -out apache.crt", self.logh)
      with open(Installer.CONFPATH, "w") as conf:
        conf.write(Installer.HTTPSSLCONF % ({"serverip":serverip}))
      os.chdir(CURDIR)
    else:
      with open(Installer.CONFPATH, "w") as conf:
        conf.write(Installer.HTTPDCONF % ({"serverip":serverip}))
