# Copyright (c) 2018  University of Houston

from __future__ import absolute_import

import signal

from . import ApplicationManager, InstallableApplication, LocalApplication

class ARPing(LocalApplication):
  # Intended grat ARP: arping -bU -c 3 -I eth1 myIP
  # Intended ARP probe: arping -bUD -c 3 -I eth1 myIP or arping -bAD -c 3 -I eth1 myIP
  def __init__ (self, dest, interface, count = None, broadcast = True, source = None, unsolicited = None, dadmode = False, deadline = None, logh = None):
    super(ARPing, self).__init__("ARPing", logh) 
    self._binpath_alpine = "nohup /usr/sbin/arping"
    self.dest = dest                 # target IP
    self.interface = interface       # network interface "-I"
    self.count = count               # number of ARP packets "-c"
    self.broadcast = broadcast       # MAC broadcast only "-b"
    self.source = source             # source network address, "-s", if blank source IP = 0.0.0.0
    self.unsolicited = unsolicited   # "-U", source IP = destination IP -U or -A, REQUEST, REPLY, or None
    self.dadmode = dadmode
    self.deadline = deadline

  def start (self, block = False):
    cmd_parts = [self.binpath]
    if self.count: cmd_parts.append("-c %d" % (self.count))
    if self.broadcast: cmd_parts.append("-b")
    if self.source: cmd_parts.append("-s %s" % (self.source))
    if self.unsolicited == "REQUEST": cmd_parts.append("-U")
    if self.unsolicited == "REPLY": cmd_parts.append("-A")
    if self.dadmode: cmd_parts.append("-D")
    if self.deadline: cmd_parts.append("-w %s" % (self.deadline))

    cmd_parts.append("-I %s %s" % (self.interface, self.dest)) # interface and dest MUST COME LAST
    if block:
      return self.call(" ".join(cmd_parts), block = block)
    else:
      self.pid = self.call(" ".join(cmd_parts), block = block)
      ApplicationManager.writePID(self)
      return self.pid

  def stop (self):
    self.kill(signal.SIGTERM)


class Installer(InstallableApplication):
  def __init__ (self, logh):
    super(Installer, self).__init__("ARPing", logh)

  def install_alpine (self):
    self.call("%s iputils" % (self.APK))
