# Copyright (c) 2016-2018  University of Houston

from __future__ import absolute_import

import os
import signal
import stat

from . import LocalApplication, ApplicationManager, InstallableApplication
from ..util import call

class iPerfC(LocalApplication):
  SCRDIR = "/tmp"
  SCRIPT = """#!/bin/bash
date +%%s.%%N
%(binpath)s -c %(endpoint)s -u -b %(bitrate)dm -t %(duration)d > /dev/null
date +%%s.%%N"""
  # TODO allow for non-megabit values for bitrate

  def __init__ (self, endpoint, bitrate, duration, logh = None):
    super(iPerfC, self).__init__("iperfc", logh)
    self._binpath_ubuntu = "/usr/bin/iperf"
    self._binpath_alpine = "nohup /usr/bin/iperf"

    self.endpoint = endpoint
    self.bitrate = bitrate
    self.duration = duration

  def start (self, block = False):
    scriptpath = "%s/iperfc-%d.sh" % (iPerfC.SCRDIR, self._id)

    with open(scriptpath, "w+") as iperfscript:
      iperfscript.write(iPerfC.SCRIPT % ({"endpoint" : self.endpoint, "bitrate" : self.bitrate,
                                          "duration" : self.duration, "binpath" : self.binpath}))
    st = os.stat(scriptpath)
    os.chmod(scriptpath, st.st_mode | stat.S_IEXEC)

    if block: 
      return self.call(scriptpath, block = block)
    else: 
      self.pid = self.call(scriptpath, block = block)
      ApplicationManager.writePID(self)

  def stop (self):
    self.kill(signal.SIGTERM)


class iPerfS(LocalApplication):
  SCRDIR = "/tmp"
  SCRIPT = """#!/bin/bash
%(binpath)s -s -u -p %(port)d > /dev/null"""

  def __init__ (self, port = 5001, logh = None):
    super(iPerfS, self).__init__("iperfs", logh)
    self._binpath_ubuntu = "/usr/bin/iperf"
    self._binpath_alpine = "nohup /usr/bin/iperf"
    self.port = port

  def start (self, block = False):
    scriptpath = "%s/iperfs-%d.sh" % (iPerfS.SCRDIR, self._id)

    with open(scriptpath, "w+") as iperfscript:
      iperfscript.write(iPerfS.SCRIPT % ({"port" : self.port, "binpath" : self.binpath}))
    st = os.stat(scriptpath)
    os.chmod(scriptpath, st.st_mode | stat.S_IEXEC)

    if block:
      return self.call(scriptpath, block = block)
    else:
      self.pid = self.call(scriptpath, block = block)
      ApplicationManager.writePID(self)

  def stop (self):
    self.kill(signal.SIGTERM)


class Installer(InstallableApplication):
  def __init__ (self, logh):
    super(Installer, self).__init__("iperf", logh)

  def install_ubuntu (self):
    call("%s iperf" % (self.APT), self.logh)
