# Copyright (c) 2016-2018  University of Houston

from __future__ import absolute_import

import os
import signal
import stat

from . import LocalApplication, ApplicationManager, InstallableApplication
from ..util import call

class Netcat(LocalApplication):
  SCRDIR = "/tmp"
  SCRIPT = """#!/bin/bash
%(binpath)s -%(flags)s %(port)d > /dev/null
"""

  def __init__ (self, port, proto = "tcp", logh = None):
    super(Netcat, self).__init__("netcat", logh)
    self.port = port
    self.proto = proto
    self._pid = None
    self._binpath_ubuntu = "/usr/bin/nc"
    self._binpath_alpine = "/usr/bin/nc"

  def start (self):
    scriptpath = "%s/nc-%d.sh" % (Netcat.SCRDIR, self._id)

    # Write our script out and make it executable
    with open(scriptpath, "w") as ncscript:
      ncscript.write(Netcat.SCRIPT % ({"flags" : self.flags, "port" : self.port,
                                       "binpath" : self.binpath}))
    st = os.stat(scriptpath)
    os.chmod(scriptpath, st.st_mode | stat.S_IEXEC)

    self.pid = self.call(scriptpath, block = False)
    ApplicationManager.writePID(self)

  @property
  def flags (self):
    if self.proto == "tcp":
      return "dtkln"
    elif self.proto == "udp":
      return "dukln"

  def stop (self):
    self.kill(signal.SIGTERM)


class Installer(InstallableApplication):
  def __init__ (self, logh):
    super(Installer, self).__init__("netcat", logh)

  def install_ubuntu (self):
    call("%s netcat-openbsd" % (self.APT), self.logh)

  def install_alpine (self):
    self.call("%s netcat-openbsd" % (self.APK))
