# Copyright (c) 2018  University of Houston

from __future__ import absolute_import

import signal

from . import ApplicationManager, InstallableApplication, LocalApplication

class Ping(LocalApplication):
  def __init__ (self, dest, count = None, timestamp = False, interval = None, size = None, logh = None):
    super(Ping, self).__init__("Ping", logh) 
    self._binpath_alpine = "nohup /bin/ping"
    self.dest = dest
    self.count = count
    self.timestamp = timestamp
    self.interval = interval
    self.size = size

  def start (self, block = False):
    cmd_parts = [self.binpath]
    if self.count: cmd_parts.append("-c %d" % (self.count))
    if self.timestamp: cmd_parts.append("-D")
    if self.interval: cmd_parts.append("-i %f" % (self.interval))
    if self.size: cmd_parts.append("-s %d" % (self.size))
    cmd_parts.append(self.dest) # dest MUST COME LAST
    if block:
      return self.call(" ".join(cmd_parts), block = block)
    else:
      self.pid = self.call(" ".join(cmd_parts), block = block)
      ApplicationManager.writePID(self)
      return self.pid

  def stop (self):
    self.kill(signal.SIGTERM)


class Installer(InstallableApplication):
  def __init__ (self, logh):
    super(Installer, self).__init__("ping", logh)

  def install_alpine (self):
    self.call("%s iputils" % (self.APK))
