# Copyright (c) 2016-2018  University of Houston

from __future__ import absolute_import

import os.path
import signal

from . import LocalApplication, ApplicationManager, InstallableApplication

class TCPDump(LocalApplication):
  def __init__ (self, ofpath, interface, snaplen = 200, filter = None, opts = None, logh = None):
    super(TCPDump, self).__init__("tcpdump", logh)
    self._binpath_ubuntu = "/usr/sbin/tcpdump"
    self._binpath_alpine = "nohup /usr/sbin/tcpdump"
    self._binpath_raspbian = "/usr/sbin/tcpdump"

    self.interface = interface
    self.snaplen = snaplen
    self.ofpath = ofpath
    self.options = opts
    self.filter = filter
    self._pid = -1

  def start (self):
    fname = self.ofpath
    IDX = 0
    while os.path.exists(fname):
      fname = "%s.%d" % (self.ofpath, IDX)
      IDX += 1
    cmdl = [self.binpath, "-i", self.interface, "-s", "%d" %(self.snaplen), "-w", fname]
    if self.options: cmdl.append(self.options)
    if self.filter: cmdl.append(self.filter)

    self.pid = self.call(" ".join(cmdl), block = False)
    ApplicationManager.writePID(self)
    return self.pid

  def stop (self):
    self.kill(signal.SIGTERM)


class Installer(InstallableApplication):
  def __init__ (self, logh):
    super(Installer, self).__init__("tcpdump", logh)

  def install_ubuntu (self):
    self.call("%s tcpdump" % (self.APT))

  def install_alpine (self):
    self.call("%s tcpdump" % (self.APK))
