# Copyright (c) 2018  University of Houston

from __future__ import absolute_import

import json
import os
import time

from .core.bases import ClientApplication

class ARPing(ClientApplication):
  def __init__ (self, host):
    super(ARPing, self).__init__("ARPing", host)

  def run_block (self, dest, interface = None, count = None, broadcast = True, source = None, unsolicited = None, dadmode = False, deadline = None):
    args = {"dest" : dest, "interface" : interface, "count" : count, "broadcast" : broadcast, "source" : source, \
            "unsolicited" : unsolicited, "dadmode": dadmode, "deadline" : deadline}
    cmdl = """apprunner ARPing '%s' -1""" % (json.dumps(args))
    self.host.run(cmdl, timeout = count*10)

  def _run (self, queue, dest, interface = None, count = None, broadcast = True, source = None, unsolicited = None, dadmode = False, deadline = None):
    args = {"dest" : dest, "interface" : interface, "count" : count, "broadcast" : broadcast, "source" : source, \
            "unsolicited" : unsolicited, "dadmode": dadmode, "deadline" : deadline}
    cmdl = """nohup apprunner ARPing '%s' -1 & echo $!""" % (json.dumps(args))
    self.pid = int(self.host.run(cmdl)[0])

    if deadline:
      timeout = count * deadline * 10
      exp_wait = wait = count * deadline
    else:
      timeout = count * 10
      exp_wait = wait = count
      
    time.sleep(exp_wait)

    while wait < timeout:
      if self.host.checkPID(self.pid):
        time.sleep(1)
        wait += 1
      else:
        queue.put({"v" : 1, "success" : True, "pid" : os.getpid()})
        break
    if wait >= timeout:
      queue.put({"v" : 1, "success" : False, "pid" : os.getpid()})

    return

  def _install (self, queue):
    # TODO: some install method here using uhexp instmng when ready
    queue.put({"v" : 1, "success" : True, "pid" : os.getpid()})
    return
