# Copyright (c) 2018  University of Houston

from __future__ import absolute_import

from .core.bases import ClientApplication

class SysInfo(ClientApplication):
  def __init__ (self, host):
    super(SysInfo, self).__init__("SysInfo", host)
    self._data = {}

  def getOS (self):
    key = "os"
    if key not in self._data:
      found = False
      data = self.host.run("cat /etc/os-release")
      for line in data:
        if line.startswith("ID") and not found:
          name = line.split("=")[1].strip()
          self._data[key] = name.lower()
          self.host.os = name.lower()
          found = True
      if not found:
        data = self.host.run("cat /etc/lsb-release")
        for line in data:
          if line.startswith("DISTRIB_ID") and not found:
            name = line.split("=")[1].strip()
            self.host.os = name.lower()
            self._data[key] = name.lower()
    return self._data[key]

  def getIPv4Addresses (self, interface):
    key = "ipv4-%s" % (interface)
    if key not in self._data:
      cmdl = """ip addr show %s | grep inet""" % (interface)
      data = self.host.run(cmdl, timeout = 5)
      v4addrs = []
      v6addrs = []
      for row in data:
        parts = row.split()
        if len(parts) < 3:
          continue
        if parts[0] == "inet":
          v4addrs.append(parts[1])
        elif parts[1] == "inet6":
          v6addrs.append(parts[1])
      self._data[key] = v4addrs
      self._data["ipv6-%s" % (interface)] = v6addrs
    return self._data[key]

  def getARPTable (self):
    data = self.host.run("cat /proc/net/arp", timeout = 5)
    return data
