# Copyright (c) 2017-2018  University of Houston

from __future__ import absolute_import

import json
import os

from .core.bases import ClientApplication


class TCPDump(ClientApplication):
  def __init__ (self, host, logh = None):
    super(TCPDump, self).__init__("TCPDump", host)

  def start (self, interface = "eth1", snaplen = 200, ofpath = "/exp-data/monitor.pcap", opts = None):
    data = {"ofpath" : ofpath, "interface" : interface, "snaplen" : snaplen}
    if opts:
      data = {"ofpath" : ofpath, "interface" : interface, "snaplen" : snaplen, "opts" : opts}
    if self.host.os == "alpine":
      cmdl = """nohup apprunner TCPDump '%s' & echo $!""" % (json.dumps(data))
      out = self.host.run(cmdl)
      for line in out:
        if len(line) >= 1 and len(line) < 8:
          self.logh.debug("Possible PID line: %s" % (line))
          self.pid = int(line)
          break
      cmdl = """kill -SIGUSR1 %d""" %(self.pid)
      self.host.run(cmdl)
    else:
      cmdl = """nohup apprunner TCPDump '%s' -2 & echo $!""" % (json.dumps(data))
      self.pid = int(self.host.run(cmdl)[0])
#      cmdl = "kill -SIGUSR1 %d" % (self.pid)
#      self.host.run(cmdl)

    
  def stop (self):
    if self.pid:
      cmdl = "kill -TERM %d" % (self.pid)
      self.host.run(cmdl)
    else:
      self.logh.warning("requested to kill absent process")

  def _install (self, queue):
    # TODO: some install method here using uhexp instmng when ready
    queue.put({"v" : 1, "success" : True, "pid" : os.getpid()})
    return
