# Copyright (c) 2015-2018  University of Houston

from __future__ import absolute_import, print_function

import json
import logging
import os
import os.path
import subprocess
import sys
import time

LOGFORMAT = "%(asctime)s [%(levelname)s] - %(message)s"

PASSWD_ASK = (-1, -1)

def getLog (path):
  logger = logging.getLogger(os.path.basename(path).split(".")[0])

  handler = logging.FileHandler(path)
  handler.setLevel(logging.DEBUG)
  handler.setFormatter(logging.Formatter(LOGFORMAT))
  logger.addHandler(handler)
  logger.setLevel(logging.DEBUG)
  return logger

def enableSSHLog (host):
  logger = logging.getLogger("ssh.%s" % (host))
  handler = logging.FileHandler("ssh-%s.log" % (host))
  handler.setLevel(logging.DEBUG)
  handler.setFormatter(logging.Formatter(LOGFORMAT))
  logger.addHandler(handler)
  logger.setLevel(logging.DEBUG)

def enableParamikoLog ():
  logger = logging.getLogger("paramiko.transport")

  handler = logging.FileHandler("paramiko.log")
  handler.setLevel(logging.DEBUG)
  handler.setFormatter(logging.Formatter(LOGFORMAT))
  logger.addHandler(handler)
  logger.setLevel(logging.DEBUG)

class KeyAdapter(logging.LoggerAdapter):
  def __init__ (self, key, logger):
    logging.LoggerAdapter.__init__(self, logger, {"key" : key})

  def process (self, msg, kwargs):
    key = self.extra["key"]
    return ("[%s] %s" % (key, msg), kwargs)

def checkAndDie (token_list):
  failed = False
  for token in token_list:
    if not token.success:
      failed = True
      print("[FAIL] %s: %s" % (token.hostname, token.command))
  if failed:
    sys.exit(1)


def call (cmd, logh = None, block = True):
  if logh:
    logh.info(cmd)
  p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
  if block:
    (sout, serr) = p.communicate()
    if logh:
      for line in serr.split("\n"):
        if line.strip():
          logh.error(line)
      for line in sout.split("\n"):
        if line.strip():
          logh.info(line)
    return p.returncode
  else:
    return p.pid


class APIEncoder(json.JSONEncoder):
  def default (self, obj):
    if hasattr(obj, "__json__"):
      return obj.__json__()
    elif isinstance(obj, set):
      return list(obj)
    else:
      return json.JSONEncoder.default(self, obj)

def dumps (data):
  return json.dumps(data, cls=APIEncoder)

loads = json.loads
